#include "geotraceman.h"
#include "convert.h"
#include <stdio.h>
#include <math.h>


float angle(x1,y1,x2,y2)
        float x1,y1,x2,y2;
{
    float temp;

    if((x2-x1) == 0.0)
        temp = PI / 2.0;
    else
        temp = atan(fabs((y2-y1) / (x2-x1)));
    if((y2-y1 >= 0.0) && (x2-x1 < 0.0))
        temp = PI - temp;
    else if((y2-y1 < 0.0) && (x2-x1 < 0.0))
        temp = PI + temp;
    else if((y2-y1 < 0.0) && (x2-x1 >= 0.0))
        temp = 2.0*PI - temp;
    return temp;
}


determine_transformation(p1,p2,a,b,c,d,radius)
float p1[],p2[];
float *a,*b,*c,*d;
float radius;
{
    float q1[3],q2[3],n[3];
    float rad2,rad3;

    /*
    printf("entering determine_transformation\n");
    */

    /* determine coords of p1 and p2 */

    q1[0] = radius * cos(p1[1]) * sin(p1[0]);
    q1[1] = radius * sin(p1[1]);
    q1[2] = radius * cos(p1[1]) * cos(p1[0]);
    q2[0] = radius * cos(p2[1]) * sin(p2[0]);
    q2[1] = radius * sin(p2[1]);
    q2[2] = radius * cos(p2[1]) * cos(p2[0]);

    /*
    printf("3d coords: %f %f %f -> %f %f %f\n",
    		q1[0],q1[1],q1[2],q2[0],q2[1],q2[2]);
    */

    /* determine normal to Op1 and Op2 - take cross product */

    n[0] = q1[1] * q2[2] - q1[2] * q2[1];
    n[1] = q1[2] * q2[0] - q1[0] * q2[2];
    n[2] = q1[0] * q2[1] - q1[1] * q2[0];

    /*
    printf("3d coords of normal: %f %f %f\n", n[0],n[1],n[2]);
    printf("3d coords: %f %f %f -> %f %f %f\n",
    		q1[0],q1[1],q1[2],q2[0],q2[1],q2[2]);
    */

    /* 
       determine transformation angles by rotating to get normal along
       the x axis and Op1 along the y axis
    */
    /*
       first rotate about the z and y axes to get normal along x axis
       and the plane O,P1,P2 in the yz plane
    */


    /* rotate about y axis by - *a */
    *a = angle(0.0,0.0,n[0],n[2]);
    /*
    printf("a: %f\n",*a);
    */

    rotate_y_axis(n,-(*a));
    rotate_y_axis(q1,-(*a));
    rotate_y_axis(q2,-(*a));

    /*
    printf("3d coords of normal after z rot: %f %f %f\n", n[0],n[1],n[2]);
    printf("3d coords: %f %f %f -> %f %f %f\n",
    		q1[0],q1[1],q1[2],q2[0],q2[1],q2[2]);
    */

    /* rotate about z axis by - *b */
    *b = angle(0.0,0.0,n[0],n[1]);
    /*
    printf("b: %f\n",*b);
    */

    rotate_z_axis(n,-(*b));
    rotate_z_axis(q1,-(*b));
    rotate_z_axis(q2,-(*b));

    /*
    printf("3d coords of normal after y rot: %f %f %f\n", n[0],n[1],n[2]);
    printf("3d coords: %f %f %f -> %f %f %f\n",
    		q1[0],q1[1],q1[2],q2[0],q2[1],q2[2]);
    */

    /* rotate q1 about x axis to put q1 along y axis */
    *c = angle(0.0,0.0,q1[1],q1[2]);

    /*
    printf("c: %f\n",*c);
    */

    rotate_x_axis(n,-(*c));  /* not needed - just a check */
    rotate_x_axis(q1,-(*c));
    rotate_x_axis(q2,-(*c));

    /*
    printf("3d coords of normal after x rot: %f %f %f\n", n[0],n[1],n[2]);
    printf("3d coords: %f %f %f -> %f %f %f\n",
    		q1[0],q1[1],q1[2],q2[0],q2[1],q2[2]);
    */

    /*  determine the angle between the two points - take dot product */
    *d = acos(
	(q1[1]*q2[1]+q1[2]*q2[2])
	/
	(
	 sqrt(sqr(q1[1])+sqr(q1[2]))
	    *
	 sqrt(sqr(q2[1])+sqr(q2[2]))
	 ));
    /*
    printf("d: %f\n",*d);

    printf("leaving determine_transformation ");
    */
}
    
rotate_z_axis(n,angle)
float angle;
float *n;
{
    float temp;
    float sine, cosine;

    sine = sin(angle);
    cosine = cos(angle);
    temp = n[0] * cosine - n[1] * sine;
    n[1] = n[0] * sine + n[1] * cosine;
    n[0] = temp;
}

rotate_y_axis(n,angle)
float angle;
float *n;
{
    float temp;
    float sine, cosine;

    sine = sin(angle);
    cosine = cos(angle);
    temp = n[0] * cosine - n[2] * sine;
    n[2] = n[0] * sine + n[2] * cosine;
    n[0] = temp;
}

rotate_x_axis(n,angle)
float angle;
float *n;
{
    float temp;
    float sine, cosine;

    sine = sin(angle);
    cosine = cos(angle);
    temp = n[1] * cosine - n[2] * sine;
    n[2] = n[1] * sine + n[2] * cosine;
    n[1] = temp;
}

ThreeD_line(p1,p2)
    float p1[2],p2[2];
{
    float a,b,c,d;
    float q[3];
    float ang;
    float par1,par2,height;   /* for parabolic flight */
    int xp,yp;
    int v[2];
    float coord[2];
    float x_scale,y_scale;
    int x_cent,y_cent,tmp;
    long tmp_coord[2];
    float radius;

    radius = 384.0;
    x_cent = 512.0;
    y_cent = 512.0;

    /*
    printf("initial lats and longs: %f %f -> %f %f\n",p1[0],p1[1],p2[0],p2[1]);
    */


    p1[0] = deg_to_rad(p1[0]);
    p1[1] = deg_to_rad(p1[1]);
    p2[0] = deg_to_rad(p2[0]);
    p2[1] = deg_to_rad(p2[1]);
 
    /*
    printf("initial lats and longs: %f %f -> %f %f\n",p1[0],p1[1],p2[0],p2[1]);
    */

    determine_transformation(p1,p2,&a,&b,&c,&d,radius);

    /*
    printf("rotation angles: %f %f %f\n",a,b,c,d);
    */
    

    /*
       determine parameters of a parabola: y=ax^2+h
       where h is the height of the peak
    */

    par2 = radius / 5.0;
    par1 = - 4.0 * par2 / sqr(d); 

    color(Map_Color_Trace);
    bgnline();

   for(ang=0.0;ang<=d;ang+=d/40.0){
	height = sqr(ang - d/2.0) * par1 + par2;
	q[0] = 0.0;
	q[1] = (radius + height) * cos(ang);
	q[2] = (radius + height) * sin(ang);
	rotate_x_axis(q,c);
	rotate_z_axis(q,b);
	rotate_y_axis(q,a);

    	/* simply plot the x and y coords as a point after scaling */
    	/* scaling */

    	coord[0] = q[0] + x_cent;
    	coord[1] = q[1] + y_cent;


	/* if is is not on the back and inside the circle */ 
	/* this is the only case whn not to draw the line*/

	if (!((sqr(q[0]) + sqr(q[1]) < sqr(radius)) && (q[2] > 0.0)))
	{

		fflush(stdout);
		v2f(coord);
	}
	
    }
    endline();
    swapbuffers();

}
