/***********************************************************
        	      (C) Copyright 1993
		Curtin University of Technology
		    Perth Western Australia



 This software is provided as is without any warranty express
 or implied.  Curtin University make no representation as to its
 suitability for any purpose whatsoever.

 Permission to use, modify or copy this software and its
 documentation is hereby granted to the source license holder.
 It is an express condition of such permission that the user
 accepts all responsibility for the use of the software and
 further that:

 1.    this notice appears in any copies of the software and the
       supporting documentation; and

 2.    the name of Curtin University not be used in any
       advertising or publicity pertaining to the software nor
       used to endorse or promote any product derived from this
       software without prior specific written permission.

 3.    the software in source form may not be given to any
       party other than those listed on the source license.

******************************************************************/
static char sccsid[] = "@(#)widgets.c	1.4	6/10/93";

/*
 * widgets.c
 *
 * Special thanks to George Furguson for the dynamic widget creation stuff.
 *
 */

#include "geotraceman.h"

#define	NUL		'\0'
#define ISSPACE(c)	((c)==' '||(c)=='\t'||(c)=='\n')

/* functions defined here */
void    fail0 (), fail1 (), doPendingEvents (), setText ();
void	settick(), unsettick();
void    setLabel ();
void    DestroyPopupPrompt (), DestroyPopup ();
void	RemoveTopshell();
void    initWidgetsFromString ();
void    GetStringDialog ();
void 	DisplayMessage();
void	ZoomPopup();
void	Ok();
void    StartTrace();
/*	-	-	-	-	-	-	-	-	-     */

void (*stringdialogfunc)();

void
fail0 (str)
char   *str;
{
    fprintf (stderr, "%s: %s\n", prog_name, str);
    XtDestroyApplicationContext (appContext);
    exit (1);
}

void
fail1 (fmt, arg)
char   *fmt, *arg;
{
    char    buf[256];

    sprintf (buf, fmt, arg);
    fail0 (buf);
}

void
setTick (widgetstr)
char   *widgetstr;
{
    Widget  w;

    if ((w = XtNameToWidget (toplevel, widgetstr)) != NULL)
        XtVaSetValues (w, XtNleftBitmap, tick_pixmap, NULL);
}

void
unsetTick (widgetstr)
char   *widgetstr;
{
    Widget  w;

    if ((w = XtNameToWidget (toplevel, widgetstr)) != NULL)
        XtVaSetValues (w, XtNleftBitmap, None, NULL);
}

void Ok(widget, event, params, num_params)
Widget widget;
XEvent *event;
String *params;
Cardinal *num_params;
{
        Widget dialog = XtParent(widget);

        (*stringdialogfunc)(widget, (XtPointer)dialog, (XtPointer)NULL);
}

void
doPendingEvents ()
{
    while (XtAppPending (appContext))
	XtAppProcessEvent (appContext, XtIMAll);
}

void
setText (item, text)
Widget  item;
char   *text;
{
    if (item != NULL)
	XtVaSetValues (item, XtNstring, text, NULL);
    else
	fail0("setText: item NULL");
}

void
setLabel (item, text)
Widget  item;
char   *text;
{
    if (item != NULL)
	XtVaSetValues (item, XtNlabel, text, NULL);
    else
	fail0("setLabel: item NULL");
}

void DestroyPopupPrompt(widget, client_data, call_data)
Widget  widget;
XtPointer client_data, call_data;
{
        Widget popup = XtParent((Widget)client_data);
        XtDestroyWidget(popup);
}
 
void DestroyPopup(w)
Widget w;
{
        DestroyPopupPrompt((Widget)NULL, (XtPointer)w, (XtPointer)NULL);
}

void RemoveTopshell(w)
Widget w;
{
        while(XtClass(w) != transientShellWidgetClass)
            w = XtParent(w);

        XtDestroyWidget(w);
}

void
StartTrace(widget, client_data, call_data)
Widget widget;
XtPointer client_data, call_data;
{
        Widget dialog = (Widget)client_data;
	Widget tmpWidget;
        char *hostp = XawDialogGetValueString(dialog);
	Trace_Struct *tracep;
	struct hostent *host_ent;
	unsigned long host_addr;

	DestroyPopupPrompt(NULL, (XtPointer)client_data, (XtPointer)NULL);

	/* check if host is entered by name or number */
	if (isdigit(hostp[0]) == 0) {
		host_ent = gethostbyname(hostp);
	}
	else {
		host_addr = inet_addr(hostp);
		host_ent = gethostbyaddr(&host_addr, 4, AF_INET);
	}

	if (host_ent == NULL) {
		DisplayMessage(toplevel, "Error", "Can't contact host");
		return;
	}

        if ((ts.tracepid = fork_trace(host_ent->h_name, &ts.tracesd)) != 0) {
		ts.tracing = True;
		if (ts.numtrace == 0)
			create_trace(&(ts.traces));
		else {
			go_to_trace((ts.numtrace -1), &tracep);
			create_trace(&tracep->next_trace);
		}
        	XtAppAddWorkProc (appContext,(XtWorkProc) readtracepipeanddraw, ts.tracesd);

	/* update trace statistics */
	if ((tmpWidget = XtNameToWidget( toplevel, "*tstatusnLabel")) == NULL)
		fail0("No such widget *tstatusnLabel");
        setLabel(tmpWidget , "tracing");
	}
}
 

void GetStringDialog(widget, dialogname, prompt, func)
Widget widget;
char *dialogname;
char *prompt;
void (*func)();
{
        Widget          popup, dialog, tmpWidget, label, value, ok, cancel;
        Position        x, y;
        Dimension       width, height;
 
        /* create the widgets */
        initWidgetsFromString(appResources.stringDialogWidgets,
                        ".stringDialogWidgets",
                        widget);
        popup = XtNameToWidget(widget, "*stringShell");
        dialog = XtNameToWidget(popup, "*stringWidget");
	label = XtNameToWidget(popup, "*stringWidget.label");
	value = XtNameToWidget(popup, "*stringWidget.value");

        stringdialogfunc = func;
        XawDialogAddButton(dialog, "Ok", func, (XtPointer)dialog);
        XawDialogAddButton(dialog, "Cancel", DestroyPopupPrompt, (XtPointer)dialog);

	ok = XtNameToWidget(popup, "*stringWidget.Ok");
	cancel = XtNameToWidget(popup, "*stringWidget.Cancel");

        /* set the position for the widget */
        XtVaGetValues(widget, XtNwidth, &width, XtNheight, &height, NULL);
        XtTranslateCoords(widget, (Position) (width / 2), (Position) (height / 2), &x, &y);
        XtVaSetValues(popup, XtNx, x, XtNy, y, NULL);

	if (nm_display_type == MONO) {
	        XtVaSetValues(dialog, XtNforeground, MONO_WHITE, XtNbackground, MONO_BLACK, NULL);
       	 	XtVaSetValues(label, XtNforeground, MONO_WHITE, XtNbackground, MONO_BLACK, NULL);
        	XtVaSetValues(value, XtNforeground, MONO_BLACK, XtNbackground, MONO_WHITE, NULL);
        	XtVaSetValues(ok, XtNforeground, MONO_WHITE, XtNbackground, MONO_BLACK, NULL);
        	XtVaSetValues(cancel, XtNforeground, MONO_WHITE, XtNbackground, MONO_BLACK, NULL);
	}
 
        if (prompt != NULL)
                setLabel(dialog, prompt);
        if (dialogname != NULL) {
                XtRealizeWidget(popup);
                XStoreName(XtDisplay(popup), XtWindow(popup), dialogname);
        }
        XtPopup(popup, XtGrabExclusive);
}


void DisplayMessage(widget, popname, message)
Widget widget;
char *popname, *message;
{
        Widget label, popup, dismiss;
        Position        x, y;
        Dimension       width, height;
        char *hier =   ".                       TransientShell  messShell \
                        messShell               Box             messBox \
                        messShell.messBox       Label           messageLabel \
                        messShell.messBox       Command         dismissButton \
                       ";

        initWidgetsFromString(hier, "", widget);

        popup = XtNameToWidget(widget, "*messShell");
        label = XtNameToWidget(popup, "*messageLabel");
        dismiss = XtNameToWidget(popup, "*dismissButton");

        setLabel(dismiss, "Dismiss");
        setLabel(label, message);
	if (nm_display_type == MONO) {
		XtVaSetValues(dismiss, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
		XtVaSetValues(label, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
	}

        if (*popname != '\0') {
                XtRealizeWidget(popup);
                XStoreName(XtDisplay(popup), XtWindow(popup), popname);
        }

        /* set the position for the widget */
        XtVaGetValues(widget, XtNwidth, &width, XtNheight, &height, NULL);
        XtTranslateCoords(widget, (Position) (width / 2), (Position) (height / 2), &x, &y);
        XtVaSetValues(popup, XtNx, x, XtNy, y, NULL);

        XtAddCallback(dismiss, "callback", (XtCallbackProc)RemoveTopshell, (XtPointer)0);
        XtPopup(popup, XtGrabNone);
}

void ZoomPopup(widget)
	Widget widget;
{
        Widget          popup, dialog, worldbm;
        Position        x, y;
        Dimension       width, height;
	Pixmap		world_pixmap;
 
        /* create the widgets */
        initWidgetsFromString(appResources.zoomboxWidgets,
                        ".zoomboxWidgets",
                        widget);
 
        /* set the position for the widget */
        XtVaGetValues(widget, XtNwidth, &width, XtNheight, &height, NULL);
        XtTranslateCoords(widget, (Position) (width / 2), (Position) (height / 2
), &x, &y);
        popup = XtNameToWidget(widget, "*zoomShell");
	worldbm = XtNameToWidget(widget, "*worldXBM");
        XtVaSetValues(popup, XtNx, x, XtNy, y, NULL);
	
	passpixmap(&world_pixmap);
	XtVaSetValues(worldbm, XtNbackgroundPixmap, world_pixmap, NULL); 
        /*if (prompt != NULL)*/
                /*setLabel(dialog, prompt);*/
        /*if (dialogname != NULL) {*/
        XtRealizeWidget(popup);
	/*XStoreName(XtDisplay(popup), XtWindow(popup), dialogname);*/
        /*}*/
 
        XtPopup(popup, XtGrabExclusive);
}

void
initWidgetsFromString (resourceStr, resourceName, shell)
char   *resourceStr, *resourceName;
Widget  shell;
{
    char    name[32], class[32], parent[256];
    char   *s, *t;
    Boolean isShell;
    WidgetClass wc;
    Widget  pw;
    Boolean usingShell;

    if ((s = resourceStr) == NULL)
	fail1 ("no widgets specified in %s resource!", resourceName);

    if (shell == (Widget) NULL) {
	usingShell = False;
	shell = toplevel;
    }
    else
	usingShell = True;

    while (*s) {

	/* skip leading whitespace */
	while (ISSPACE (*s))
	    s++;
	if (!*s)
	    break;

	/* Gather the parent widget name */
	t = parent;
	while (*s && !ISSPACE (*s))
	    *t++ = *s++;
	*t = NUL;

	/* skip whitespace */
	while (ISSPACE (*s))
	    s++;
	if (!*s)
	    fail1 ("missing widget class and name in %s resource", resourceName);

	/* Gather the class name */
	t = class;
	while (*s && !ISSPACE (*s))
	    *t++ = *s++;
	*t = NUL;

	/* skip whitespace */
	while (ISSPACE (*s))
	    s++;
	if (!*s)
	    fail1 ("missing widget name in %s resource", resourceName);

	/* Gather the widget's name */
	t = name;
	while (*s && !ISSPACE (*s))
	    *t++ = *s++;
	*t = NUL;

	/* convert class name to WidgetClass */
	if ((wc = classNameToWidgetClass (class, &isShell)) == NULL)
	    fail1 ("can't convert string \"%s\" to widgetClass", class);

	/* convert parent name to Widget */
	if (strcmp (parent, ".") == 0) {
	    pw = usingShell ? shell : toplevel;
	}
	else if ((pw = XtNameToWidget (shell, parent)) == NULL)
	    fail1 ("can't convert string \"%s\" to widget", parent);

	/* finally create the widget */
	if (isShell)
	    (void) XtCreatePopupShell (name, wc, pw, NULL, ZERO);
	else
	    (void) XtCreateManagedWidget (name, wc, pw, NULL, ZERO);
    }
}
